@@ -1,3 +1,5 @@
+# See LICENSE.incore for details
+
 ''' Parser for the log file '''
 import re
 
@@ -24,6 +26,26 @@ class instructionObject:
         self.csr = csr
         self.shamt = shamt
 
+    def __str__(self):
+        line = 'addr: '+ str(self.instr_addr) +' instr: '+ str(self.instr_name)
+        if self.rd:
+            line+= ' rd: '+ str(self.rd)
+        if self.rs1:
+            line+= ' rs1: '+ str(self.rs1)
+        if self.rs2:
+            line+= ' rs2: '+ str(self.rs2)
+        if self.rs3:
+            line+= ' rs3: '+ str(self.rs3)
+        if self.csr:
+            line+= ' csr: '+ str(self.csr)
+        if self.imm:
+            line+= ' imm: '+ str(self.imm)
+        if self.shamt:
+            line+= ' shamt: '+ str(self.shamt)
+        return line
+
+
+
 ''' Constants for extracting the bits '''
 FIRST2_MASK = 0x00000003
 OPCODE_MASK = 0x0000007f
@@ -34,34 +56,44 @@ RS2_MASK = 0x01f00000
 
 ''' Regex pattern and functions for extracting instruction and address '''
 instr_pattern_standard = re.compile('core\s+[0-9]+:\s+(?P<addr>[0-9abcdefx]+)\s+\((?P<instr>[0-9abcdefx]+)\)')
-instr_pattern_custom = re.compile(
+instr_pattern_spike = re.compile(
         '[0-9]\s(?P<addr>[0-9abcdefx]+)\s\((?P<instr>[0-9abcdefx]+)\)')
-instr_pattern_custom_xd = re.compile(
+instr_pattern_spike_xd = re.compile(
         '[0-9]\s(?P<addr>[0-9abcdefx]+)\s\((?P<instr>[0-9abcdefx]+)\)' +
         '\s(?P<regt>[xf])(?P<reg>[\s|\d]\d)\s(?P<val>[0-9abcdefx]+)'
 )
+instr_pattern_c_sail= re.compile(
+        '\[\d*\]\s\[(.*?)\]:\s(?P<addr>[0-9xABCDEF]+)\s\((?P<instr>[0-9xABCDEF]+)\)\s*(?P<mnemonic>.*)')
+instr_pattern_c_sail_regt_reg_val = re.compile('(?P<regt>[xf])(?P<reg>[\d]+)\s<-\s(?P<val>[0-9xABCDEF]+)')
 
 def extractInstruction(line, mode = 'standard'):
     ''' Function to extract the instruction code from the line
-        Check for the mode - custom or standard
+        Check for the mode - spike or standard
     '''
     instr_pattern = instr_pattern_standard
-    if mode == 'custom':
-        instr_pattern = instr_pattern_custom
+    if mode == 'spike':
+        instr_pattern = instr_pattern_spike
+    if mode == 'c_sail':
+        instr_pattern = instr_pattern_c_sail
 
     re_search = instr_pattern.search(line)
     if re_search is not None:
-        return int(re_search.group('instr'), 16)
+        if mode == 'c_sail':
+            return int(re_search.group('instr'), 16),re_search.group('mnemonic')
+        else:
+            return int(re_search.group('instr'), 16), None
     else:
-        return None
+        return None, None
 
 def extractAddress(line, mode = 'standard'):
     ''' Function to extract the address from the line
-        Check for the mode - custom or standard
+        Check for the mode - spike or standard
     '''
     instr_pattern = instr_pattern_standard
-    if mode == 'custom':
-        instr_pattern = instr_pattern_custom
+    if mode == 'spike':
+        instr_pattern = instr_pattern_spike
+    if mode == 'c_sail':
+        instr_pattern = instr_pattern_c_sail
 
     re_search = instr_pattern.search(line)
     if re_search is not None:
@@ -69,11 +101,15 @@ def extractAddress(line, mode = 'standard'):
     else:
         return 0
 
-def extractRegisterCommitVal(line):
+def extractRegisterCommitVal(line, mode):
     ''' Function to extract the register commit value
-        Only works for custom mode
+        Only works for spike mode
     '''
-    re_search = instr_pattern_custom_xd.search(line)
+    instr_pattern = instr_pattern_spike_xd
+    if mode == 'c_sail':
+        instr_pattern = instr_pattern_c_sail_regt_reg_val
+
+    re_search = instr_pattern.search(line)
     if re_search is not None:
         return (re_search.group('regt'), re_search.group('reg'), re_search.group('val'))
     else:
@@ -85,25 +121,25 @@ def extractOpcode(instr):
 
 def parseInstruction(input_line, mode, arch='rv32'):
     ''' Check if we are parsing compressed or normal instructions '''
-    instr = extractInstruction(input_line, mode)
+    instr, mnemonic = extractInstruction(input_line, mode)
 
     if instr is None:
 #        print("Skipping {}".format(input_line))
-        return
+        return None, None
 
     first_two_bits = FIRST2_MASK & instr
 
     if first_two_bits == 0b11:
-        return parseStandardInstruction(input_line, mode)
+        return parseStandardInstruction(input_line, mode, arch), mnemonic
     else:
-        return parseCompressedInstruction(input_line, mode, arch)
+        return parseCompressedInstruction(input_line, mode, arch), mnemonic
 
 def parseCompressedInstruction(input_line, mode, arch):
     ''' Parse a compressed instruction
         Args: input_line - Line from the log file
         Returns: (instr_obj)
     '''
-    instr = extractInstruction(input_line, mode)
+    instr, mnemonic = extractInstruction(input_line, mode)
 
     if instr is None:
         return None
@@ -119,12 +155,12 @@ def parseCompressedInstruction(input_line, mode, arch):
 
     return instrObj
 
-def parseStandardInstruction(input_line, mode):
+def parseStandardInstruction(input_line, mode, arch):
     ''' Parse an input line and decode the instruction
         Args: input_line - Line from the log file
         Returns: (instr_name, rd, rs1, rs2, imm)
     '''
-    instr = extractInstruction(input_line, mode)
+    instr, mnemonic = extractInstruction(input_line, mode)
 
     if instr is None:
         return None
@@ -132,7 +168,7 @@ def parseStandardInstruction(input_line, mode):
     addr = extractAddress(input_line, mode)
     opcode = extractOpcode(instr)
     try:
-        instrObj = opcodes[opcode](instr, addr)
+        instrObj = opcodes[opcode](instr, addr, arch)
     except KeyError as e:
         print("Instruction not found", hex(instr))
         return None
@@ -145,17 +181,17 @@ def twos_comp(val, bits):
         val = val - (1 << bits)
     return val
 
-def lui(instr, addr):
+def lui(instr, addr, arch):
     imm = instr >> 12
     rd = ((instr & RD_MASK) >> 7, 'x')
     return instructionObject('lui', addr, rd = rd, imm = imm)
 
-def auipc(instr, addr):
+def auipc(instr, addr, arch):
     imm = instr >> 12
     rd = ((instr & RD_MASK) >> 7, 'x')
     return instructionObject('auipc', addr, rd = rd, imm = imm)
 
-def jal(instr, addr):
+def jal(instr, addr, arch):
     imm_10_1 = (instr >> 21) & 0x000003ff
     imm_11 = (instr >> 10) & 0x00000400
     imm_19_12 = (instr & 0x000ff000) >> 1
@@ -165,14 +201,14 @@ def jal(instr, addr):
     rd = ((instr & RD_MASK) >> 7, 'x')
     return instructionObject('jal', addr, rd = rd, imm = imm)
 
-def jalr(instr, addr):
+def jalr(instr, addr, arch):
     rd = ((instr & RD_MASK) >> 7, 'x')
     rs1 = ((instr & RS1_MASK) >> 15, 'x')
     imm_11_0 = instr >> 20
     imm = twos_comp(imm_11_0, 12)
     return instructionObject('jalr', addr, rd = rd, rs1 = rs1, imm = imm)
 
-def branch_ops(instr, addr):
+def branch_ops(instr, addr, arch):
     funct3 = (instr & FUNCT3_MASK) >> 12
     rs1 = ((instr & RS1_MASK) >> 15, 'x')
     rs2 = ((instr & RS2_MASK) >> 20, 'x')
@@ -200,7 +236,7 @@ def branch_ops(instr, addr):
 
     return instrObj
 
-def load_ops(instr, addr):
+def load_ops(instr, addr, arch):
     funct3 = (instr & FUNCT3_MASK) >> 12
     rd = ((instr & RD_MASK) >> 7, 'x')
     rs1 = ((instr & RS1_MASK) >> 15, 'x')
@@ -225,7 +261,7 @@ def load_ops(instr, addr):
 
     return instrObj
 
-def store_ops(instr, addr):
+def store_ops(instr, addr, arch):
     funct3 = (instr & FUNCT3_MASK) >> 12
     rs1 = ((instr & RS1_MASK) >> 15, 'x')
     rs2 = ((instr & RS2_MASK) >> 20, 'x')
@@ -246,7 +282,7 @@ def store_ops(instr, addr):
 
     return instrObj
 
-def arithi_ops(instr, addr):
+def arithi_ops(instr, addr, arch):
     funct3 = (instr & FUNCT3_MASK) >> 12
     rd = ((instr & RD_MASK) >> 7, 'x')
     rs1 = ((instr & RS1_MASK) >> 15, 'x')
@@ -271,11 +307,17 @@ def arithi_ops(instr, addr):
     if funct3 == 0b001:
         instrObj.instr_name = 'slli'
         instrObj.imm = None
-        shamt = imm & 0x01f
+        if arch == 'rv32':
+            shamt = imm & 0x01f
+        elif arch == 'rv64':
+            shamt = imm & 0x03f
         instrObj.shamt = shamt
     if funct3 == 0b101:
         instrObj.imm = None
-        shamt = imm & 0x01f
+        if arch == 'rv32':
+            shamt = imm & 0x01f
+        elif arch == 'rv64':
+            shamt = imm & 0x03f
         instrObj.shamt = shamt
         rtype_bit = (imm >> 10) & 0x1
         if rtype_bit == 1:
@@ -312,7 +354,7 @@ def arithm_ops(instr, addr):
 
     return instrObj
 
-def arith_ops(instr, addr):
+def arith_ops(instr, addr, arch):
 
     # Test for RV32M ops
     funct7 = (instr >> 25)
@@ -355,7 +397,7 @@ def arith_ops(instr, addr):
 
     return instrObj
 
-def fence_ops(instr, addr):
+def fence_ops(instr, addr, arch):
     funct3 = (instr & FUNCT3_MASK) >> 12
 
     pred = (instr >> 20) & 0x0000000f
@@ -372,7 +414,7 @@ def fence_ops(instr, addr):
 
     return instrObj
 
-def control_ops(instr, addr):
+def control_ops(instr, addr, arch):
     funct3 = (instr & FUNCT3_MASK) >> 12
 
     # Test for ecall and ebreak ops
@@ -411,7 +453,7 @@ def control_ops(instr, addr):
 
     return instrObj
 
-def rv64i_arithi_ops(instr, addr):
+def rv64i_arithi_ops(instr, addr, arch):
     funct3 = (instr & FUNCT3_MASK) >> 12
     rd = ((instr & RD_MASK) >> 7, 'x')
     rs1 = ((instr & RS1_MASK) >> 15, 'x')
@@ -444,20 +486,20 @@ def rv64m_arithm_ops(instr, addr):
     instrObj = instructionObject('None', addr, rd = rd, rs1 = rs1, rs2 = rs2)
 
     if funct3 == 0b000:
-        instrObj.instr_name == 'mulw'
+        instrObj.instr_name = 'mulw'
     if funct3 == 0b100:
-        instrObj.instr_name == 'divw'
+        instrObj.instr_name = 'divw'
     if funct3 == 0b101:
-        instrObj.instr_name == 'divuw'
+        instrObj.instr_name = 'divuw'
     if funct3 == 0b110:
-        instrObj.instr_name == 'remw'
+        instrObj.instr_name = 'remw'
     if funct3 == 0b111:
-        instrObj.instr_name == 'remuw'
+        instrObj.instr_name = 'remuw'
 
     return instrObj
 
 
-def rv64i_arith_ops(instr, addr):
+def rv64i_arith_ops(instr, addr, arch):
 
     # Test for rv64m ops
     funct7 = (instr >> 25)
@@ -517,7 +559,7 @@ rv64a_instr_names = {
         0b11100: 'amomaxu.d'
 }
 
-def rv64_rv32_atomic_ops(instr, addr):
+def rv64_rv32_atomic_ops(instr, addr, arch):
     funct5 = (instr >> 27) & 0x0000001f
     funct3 = (instr & FUNCT3_MASK) >> 12
     rd = ((instr & RD_MASK) >> 7, 'x')
@@ -550,7 +592,7 @@ def rv64_rv32_atomic_ops(instr, addr):
 
         return instrObj
 
-def flw_fld(instr, addr):
+def flw_fld(instr, addr, arch):
     rd = ((instr & RD_MASK) >> 7, 'f')
     rs1 = ((instr & RS1_MASK) >> 15, 'x')
     funct3 = (instr & FUNCT3_MASK) >> 12
@@ -565,7 +607,7 @@ def flw_fld(instr, addr):
 
     return instrObj
 
-def fsw_fsd(instr, addr):
+def fsw_fsd(instr, addr, arch):
     imm_4_0 = (instr & RD_MASK) >> 7
     imm_11_5 = (instr >> 25) << 5
     imm = twos_comp(imm_4_0 + imm_11_5, 12)
@@ -583,7 +625,7 @@ def fsw_fsd(instr, addr):
 
     return instrObj
 
-def fmadd(instr, addr):
+def fmadd(instr, addr, arch):
     rd = ((instr & RD_MASK) >> 7, 'f')
     rm = (instr >> 12) & 0x00000007
     rs1 = ((instr & RS1_MASK) >> 15, 'f')
@@ -602,7 +644,7 @@ def fmadd(instr, addr):
 
     return instrObj
 
-def fmsub(instr, addr):
+def fmsub(instr, addr, arch):
     rd = ((instr & RD_MASK) >> 7, 'f')
     rm = (instr >> 12) & 0x00000007
     rs1 = ((instr & RS1_MASK) >> 15, 'f')
@@ -621,7 +663,7 @@ def fmsub(instr, addr):
 
     return instrObj
 
-def fnmsub(instr, addr):
+def fnmsub(instr, addr, arch):
     rd = ((instr & RD_MASK) >> 7, 'f')
     rm = (instr >> 12) & 0x00000007
     rs1 = ((instr & RS1_MASK) >> 15, 'f')
@@ -640,7 +682,7 @@ def fnmsub(instr, addr):
 
     return instrObj
 
-def fnmadd(instr, addr):
+def fnmadd(instr, addr, arch):
     rd = ((instr & RD_MASK) >> 7, 'f')
     rm = (instr >> 12) & 0x00000007
     rs1 = ((instr & RS1_MASK) >> 15, 'f')
@@ -659,7 +701,7 @@ def fnmadd(instr, addr):
 
     return instrObj
 
-def rv32_rv64_float_ops(instr, addr):
+def rv32_rv64_float_ops(instr, addr, arch):
     rd = ((instr & RD_MASK) >> 7, 'f')
     rm = (instr >> 12) & 0x00000007
     rs1 = ((instr & RS1_MASK) >> 15, 'f')
@@ -898,47 +940,62 @@ def rv32_rv64_float_ops(instr, addr):
 
 ''' Compressed Instruction Parsing Functions '''
 C_FUNCT3_MASK = 0xe000
-C_OP2_MASK = 0x0003
-C_RDPRIME_MASK = 0x001C
-C_RS2PRIME_MASK = 0x001C
-C_RS1PRIME_MASK = 0x0380
-C_UIMM_5_3_MASK = 0x1C00
-C_UIMM_7_6_MASK = 0x0060
-C_UIMM_6_MASK = 0x0020
-C_UIMM_2_MASK = 0x0040
+C0_OP2_MASK = 0x0003
+C0_RDPRIME_MASK = 0x001C
+C0_RS2PRIME_MASK = 0x001C
+C0_RS1PRIME_MASK = 0x0380
+C0_UIMM_5_3_MASK = 0x1C00
+C0_UIMM_7_6_MASK = 0x0060
+C0_UIMM_6_MASK = 0x0020
+C0_UIMM_2_MASK = 0x0040
+
+C1_RD_MASK = 0x0F80
+C1_RDPRIME_MASK = 0x0380
+C1_RS1PRIME_MASK = 0x0380
+C1_RS2PRIME_MASK = 0x001C
+C1_IMM_5_MASK = 0x1000
+C1_IMM_4_0_MASK = 0x007c
+C1_IMM_17_MASK = 0x1000
+C1_IMM_16_12_MASK = 0x007c
+C1_MINOR_OP_MASK = 0x0C00
+C1_MINOR_OP2_MASK = 0x0060
 
 def get_bit(val, pos):
     return (val & (1 << pos)) >> pos
 
-def quad1(instr, addr, arch):
-    '''Parse instructions from Quad1 in the RISCV-ISA-Standard'''
+def quad0(instr, addr, arch):
+    '''Parse instructions from Quad0 in the RISCV-ISA-Standard'''
     instrObj = instructionObject(None, instr_addr = addr)
     funct3 = (C_FUNCT3_MASK & instr) >> 13
 
     # UIMM 7:6:5:3
-    uimm_5_3 = (C_UIMM_5_3_MASK & instr) >> 7
-    uimm_7_6 = (C_UIMM_7_6_MASK & instr) << 1
+    uimm_5_3 = (C0_UIMM_5_3_MASK & instr) >> 7
+    uimm_7_6 = (C0_UIMM_7_6_MASK & instr) << 1
     uimm_7_6_5_3 = uimm_5_3 + uimm_7_6
 
     # UIMM 6:5:3:2
-    uimm_2 = (C_UIMM_2_MASK & instr) >> 4
-    uimm_6 = (C_UIMM_6_MASK & instr) << 1
+    uimm_2 = (C0_UIMM_2_MASK & instr) >> 4
+    uimm_6 = (C0_UIMM_6_MASK & instr) << 1
     uimm_6_5_3_2 = uimm_6 + uimm_5_3 + uimm_2
 
     # Registers
-    rdprime = (C_RDPRIME_MASK & instr) >> 2
-    rs1prime = (C_RS1PRIME_MASK & instr) >> 7
-    rs2prime = (C_RS2PRIME_MASK & instr) >> 2
+    rdprime = (C0_RDPRIME_MASK & instr) >> 2
+    rs1prime = (C0_RS1PRIME_MASK & instr) >> 7
+    rs2prime = (C0_RS2PRIME_MASK & instr) >> 2
 
     if funct3 == 0b000:
-        instrObj.instr_name = 'c.addi4spn'
         nzuimm_3 = get_bit(instr, 5)
         nzuimm_2 = get_bit(instr, 6)
         nzuimm_9_6 = get_bit(instr, 7) + (get_bit(instr,8) << 1) + (get_bit(instr,9) << 2) + (get_bit(instr,10)<<3)
         nzuimm_5_4 = get_bit(instr, 11) + (get_bit(instr, 12) << 1)
         nzuimm = (nzuimm_2 << 2) + (nzuimm_3 << 3) + (nzuimm_9_6 << 6) + (nzuimm_5_4 << 4)
+        if nzuimm == 0 and rdprime == 0:
+            instrObj.instr_name = 'c.illegal'
+            instrObj.rd = (rdprime, 'x')
+        else:
+            instrObj.instr_name = 'c.addi4spn'
+            instrObj.rd = (8 + rdprime, 'x')
         instrObj.imm = nzuimm
-        instrObj.rd = (8 + rdprime, 'x')
         return instrObj
 
     elif funct3 == 0b001:
@@ -991,20 +1048,247 @@ def quad1(instr, addr, arch):
 
     return instrObj
 
-def quad2(instr, addr, arch):
-    '''Parse instructions from Quad2 in the RISCV-ISA-Standard'''
+def quad1(instr, addr, arch):
+    '''Parse instructions from Quad1 in the RISCV-ISA-Standard'''
     instrObj = instructionObject(None, instr_addr = addr)
+    funct3 = (C_FUNCT3_MASK & instr) >> 13
 
     # Registers
-    rdprime = (C_RDPRIME_MASK & instr) >> 7
-    rs1prime = (C_RS1PRIME_MASK & instr) >> 7
-    rs2prime = (C_RS2PRIME_MASK & instr) >> 2
+    rdprime = (C1_RDPRIME_MASK & instr) >> 7
+    rs1prime = (C1_RS1PRIME_MASK & instr) >> 7
+    rs2prime = (C1_RS2PRIME_MASK & instr) >> 2
+    rd = (C1_RD_MASK & instr) >> 7
+    rs1 = (C1_RD_MASK & instr) >> 7
+
+    imm_5 = (C1_IMM_5_MASK & instr) >> 7
+    imm_4_0 = (C1_IMM_4_0_MASK & instr) >> 2
+    imm = imm_5 + imm_4_0
+
+    imm_lui = ((C1_IMM_17_MASK & instr) >> 12) + ((C1_IMM_16_12_MASK & instr)>>2)
+    imm_j_5 = get_bit(instr, 2) << 5
+    imm_j_3_1 = get_bit(instr,3) + (get_bit(instr, 4) << 1) + (get_bit(instr,5) << 2)
+    imm_j_7 = get_bit(instr,6) << 7
+    imm_j_6 = get_bit(instr,7) << 6
+    imm_j_10 = get_bit(instr, 8) << 10
+    imm_j_9_8 = get_bit(instr, 9) + (get_bit(instr,10)<< 1)
+    imm_j_4 = get_bit(instr,11) << 4
+    imm_j_11 = get_bit(instr, 12) << 11
+    imm_j = imm_j_5 + (imm_j_3_1 << 1) + imm_j_7 + imm_j_6 + imm_j_10 + (imm_j_9_8 << 8) + imm_j_4 + imm_j_11
+
+    imm_b_5 = get_bit(instr, 2) << 5
+    imm_b_2_1 = get_bit(instr, 3) + (get_bit(instr,4) << 1)
+    imm_b_7_6 = get_bit(instr, 5) + (get_bit(instr,6) << 1)
+    imm_b_4_3 = get_bit(instr, 10) + (get_bit(instr,11) << 1)
+    imm_b_8 = get_bit(instr, 12) << 8
+    imm_b = imm_b_5 + (imm_b_2_1 << 1) + (imm_b_7_6 << 6) + (imm_b_4_3 << 3) + imm_b_8
+
+    imm_addi_5 = get_bit(instr, 2) << 5
+    imm_addi_8_7 = (get_bit(instr, 3) + (get_bit(instr, 4)<< 1) ) << 7
+    imm_addi_6 = get_bit(instr, 5) << 6
+    imm_addi_4 = get_bit(instr, 6) << 4
+    imm_addi_9 = get_bit(instr, 12) << 9
+    imm_addi = imm_addi_5 + imm_addi_8_7 + imm_addi_6 + imm_addi_4 + imm_addi_9
+
+    op = (C1_MINOR_OP_MASK & instr) >> 10
+    op2 = (C1_MINOR_OP2_MASK & instr) >> 5
+
+    if funct3 == 0:
+        instrObj.rs1 = (rs1, 'x')
+        instrObj.rd = (rd, 'x')
+        instrObj.imm = twos_comp(imm, 6)
+        if rd == 0:
+            instrObj.instr_name = 'c.nop'
+        else:
+            instrObj.instr_name = 'c.addi'
+    elif funct3 == 1:
+        if arch == 'rv32':
+            instrObj.instr_name = 'c.jal'
+            instrObj.imm = twos_comp(imm_j, 12)
+            instrObj.rd = (1, 'x')
+        elif rd !=0 :
+            instrObj.instr_name = 'c.addiw'
+            instrObj.imm = twos_comp(imm, 6)
+            instrObj.rs1 = (rs1, 'x')
+            instrObj.rd = (rd, 'x')
+    elif funct3 == 2:
+        instrObj.instr_name = 'c.li'
+        instrObj.imm = twos_comp(imm, 6)
+        instrObj.rd = (rd, 'x')
+    elif funct3 == 3:
+        if rd == 2 and imm_addi != 0:
+            instrObj.instr_name = 'c.addi16sp'
+            instrObj.rs1 = (2, 'x')
+            instrObj.rd = (2, 'x')
+            instrObj.imm = twos_comp(imm_addi, 10)
+        elif rd != 2 and imm_lui !=0:
+            instrObj.instr_name = 'c.lui'
+            instrObj.imm = imm
+            instrObj.rs1 = (rd, 'x')
+            instrObj.rd = (rd, 'x')
+    elif funct3 == 4:
+        if op == 0 and imm != 0:
+            instrObj.instr_name = 'c.srli'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.imm = imm
+        elif op == 1 and imm !=0:
+            instrObj.instr_name = 'c.srai'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.imm = imm
+        elif op == 2:
+            instrObj.instr_name = 'c.andi'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.imm = twos_comp(imm, 6)
+        elif op == 3 and op2 == 0 and imm_5 == 0:
+            instrObj.instr_name = 'c.sub'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.rs2 = (8 + rs2prime, 'x')
+        elif op == 3 and op2 == 1 and imm_5 == 0:
+            instrObj.instr_name = 'c.xor'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.rs2 = (8 + rs2prime, 'x')
+        elif op == 3 and op2 == 2 and imm_5 == 0:
+            instrObj.instr_name = 'c.or'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.rs2 = (8 + rs2prime, 'x')
+        elif op == 3 and op2 == 3 and imm_5 == 0:
+            instrObj.instr_name = 'c.and'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.rs2 = (8 + rs2prime, 'x')
+        elif op == 3 and op2 == 0 and imm_5 != 0 and arch == 'rv64':
+            instrObj.instr_name = 'c.subw'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.rs2 = (8 + rs2prime, 'x')
+        elif op == 3 and op2 == 1 and imm_5 != 0 and arch == 'rv64':
+            instrObj.instr_name = 'c.addw'
+            instrObj.rs1 = (8 + rs1prime, 'x')
+            instrObj.rd = (8 + rdprime, 'x')
+            instrObj.rs2 = (8 + rs2prime, 'x')
+    elif funct3 == 5:
+        instrObj.instr_name = 'c.j'
+        instrObj.rd = (0, 'x')
+        instrObj.imm = twos_comp(imm_j, 12)
+    elif funct3 == 6:
+        instrObj.instr_name = 'c.beqz'
+        instrObj.rs1 = (8 + rs1prime, 'x')
+        instrObj.rs2 = (0 , 'x')
+        instrObj.imm = twos_comp(imm_b, 9)
+    elif funct3 == 7:
+        instrObj.instr_name = 'c.bnez'
+        instrObj.rs1 = (8 + rs1prime, 'x')
+        instrObj.rs2 = (0, 'x')
+        instrObj.imm = twos_comp(imm_b, 9)
+
+
 
     return instrObj
 
-def quad3(instr, addr, arch):
+C2_RS2_MASK = 0x007C
+C2_RD_MASK = 0x0F80
+
+
+def quad2(instr, addr, arch):
     '''Parse instructions from Quad3 in the RISCV-ISA-Standard'''
     instrObj = instructionObject(None, instr_addr = addr)
+    funct3 = (C_FUNCT3_MASK & instr) >> 13
+
+    imm_5 = get_bit(instr, 12) << 5
+    imm_4_0 = (instr & 0x007c) >> 2
+    imm_4_3 = (instr & 0x0060) >> 2
+    imm_4 = get_bit(instr, 6) << 4
+    imm_4_2 = (instr & 0x0070) >> 2
+    imm_8_6 = (instr & 0x001C) << 5
+    imm_9_6 = (instr & 0x003C) << 5
+    imm_7_6 = (instr & 0x000C) << 4
+    imm_8_6 = (instr & 0x001C) << 4
+
+    imm_5_3 = (instr & 0x1c00) >> 7
+    imm_s_8_6 = (instr & 0x0380) >> 1
+    imm_5_2 = (instr & 0x1E00) >> 7
+    imm_s_7_6 = (instr & 0x0180) >> 1
+
+    rd = (C2_RD_MASK & instr) >> 7
+    rs1 = (C2_RD_MASK & instr) >> 7
+    rs2 = (C2_RS2_MASK & instr) >> 2
+
+    imm_slli = imm_5 + imm_4_0
+    imm_fldsp = imm_5 + imm_4_3 + imm_8_6
+    imm_lwsp = imm_5 + imm_4_2 + imm_7_6
+    imm_ldsp = imm_5 + imm_4_3 + imm_8_6
+    imm_fsdsp = imm_5_3 + imm_s_8_6
+    imm_swsp = imm_5_2 + imm_s_7_6
+
+    if funct3 == 0 and imm_slli !=0 and rd !=0:
+        instrObj.instr_name = 'c.slli'
+        instrObj.rd = (rd, 'x')
+        instrObj.rs1 = (rs1, 'x')
+        instrObj.imm = imm_slli
+    elif funct3 == 1 and arch == 'rv64':
+        instrObj.instr_name = 'c.fldsp'
+        instrObj.rd = (rd, 'f')
+        instrObj.imm = imm_fldsp
+        instrObj.rs1 = (2, 'x')
+    elif funct3 == 2 and rd != 0:
+        instrObj.instr_name = 'c.lwsp'
+        instrObj.rs1 = (2, 'x')
+        instrObj.rd = (rd, 'x')
+        instrObj.imm = imm_lwsp
+    elif funct3 == 3 and arch == 'rv32':
+        instrObj.instr_name = 'c.flwsp'
+        instrObj.rd = (rd, 'f')
+        instrObj.rs1 = (2, 'x')
+        instrObj.imm = imm_lwsp
+    elif funct3 == 3 and arch == 'rv64':
+        instrObj.instr_name = 'c.ldsp'
+        instrObj.rd = (rd, 'f')
+        instrObj.rs1 = (2, 'x')
+        instrObj.imm = imm_ldsp
+    elif funct3 == 4 and rs1 != 0 and imm_5 == 0 and rs2 == 0:
+        instrObj.instr_name = 'c.jr'
+        instrObj.rs1 = (rs1, 'x')
+        instrObj.imm = 0
+    elif funct3 == 4 and rs2!=0 and imm_5 == 0:
+        instrObj.instr_name = 'c.mv'
+        instrObj.rs2 = (rs2, 'x')
+        instrObj.rd = (rd, 'x')
+    elif funct3 == 4 and rd ==0 and rs2 == 0 and imm_5 == 32:
+        instrObj.instr_name = 'c.ebreak'
+    elif funct3 == 4 and imm_5 == 32 and rs1 !=0 and rs2 == 0:
+        instrObj.instr_name = 'c.jalr'
+        instrObj.rs1 = (rs1, 'x')
+        instrObj.rd = (1, 'x')
+    elif funct3 == 4 and imm_5 == 32 and rs2 !=0 :
+        instrObj.instr_name = 'c.add'
+        instrObj.rs1 = (rs1, 'x')
+        instrObj.rs2 = (rs2, 'x')
+        instrObj.rd = (rd, 'x')
+    elif funct3 == 5:
+        instrObj.instr_name = 'c.fsdsp'
+        instrObj.rs2 = (rs2, 'f')
+        instrObj.imm = imm_fsdsp
+        instrObj.rs1 = (2 , 'x')
+    elif funct3 == 6:
+        instrObj.instr_name = 'c.swsp'
+        instrObj.rs2 = (rs2, 'x')
+        instrObj.imm = imm_swsp
+        instrObj.rs1 = (2 , 'x')
+    elif funct3 == 7 and arch == 'rv32':
+        instrObj.instr_name = 'c.fswsp'
+        instrObj.rs2 = (rs2, 'f')
+        instrObj.rs1 = (2, 'x')
+        instrObj.imm = imm_swsp
+    elif funct3 == 7 and arch == 'rv64':
+        instrObj.instr_name = 'c.sdsp'
+        instrObj.rs2 = (rs2, 'x')
+        instrObj.imm = imm_fsdsp
+        instrObj.rs1 = (2 , 'x')
     return instrObj
 
 ''' Instruction Op-codes Dict '''
@@ -1033,7 +1317,7 @@ opcodes = {
 }
 
 c_opcodes = {
-    0b00: quad1,
-    0b01: quad2,
-    0b10: quad3
+    0b00: quad0,
+    0b01: quad1,
+    0b10: quad2
 }
